import { CoreAdmin, CoreLayoutProps, RoutesWithoutLayout } from '@rakit/core';
import { FetchContextProvider, useFetch } from '@rakit/fetch';
import { RuntimeError, Loading, ThemeProvider, NotFound } from '@rakit/joy-ui';
import { createAuthProvider } from './config/createAuthProvider';
import { createDataProvider } from './config/createDataProvider';
import { i18nProvider } from './config/i18nProvider';
import { lazy, Suspense } from 'react';
import { Route } from 'react-router-dom';

const AppLoading = () => <Loading />

const Layout = (props: CoreLayoutProps) => {
  return (
    <Suspense fallback={<Loading />}>
      {props.children}
    </Suspense>
  )
}

const SignIn = lazy(() => import("./pages/SignIn"));
const PageError = lazy(() => import("./pages/PageError"));

export default function App() {
  return (
    <FetchContextProvider>
      <ThemeProvider>
        <AppContext />
      </ThemeProvider>
    </FetchContextProvider>
  );
}

function AppContext() {
  const fetch = useFetch();

  const authProvider = createAuthProvider(fetch);
  const dataProvider = createDataProvider(fetch);

  return (
    <CoreAdmin
      authProvider={authProvider}
      // authCallbackPage={ }
      // basename={ }
      catchAll={NotFound}
      dataProvider={dataProvider}
      error={RuntimeError}
      // dashboard={<CatchAll />}
      // initialLocation="/"
      i18nProvider={i18nProvider}
      loading={AppLoading}
      layout={Layout}
      loginPage={SignIn}
      // ready={ }
      requireAuth={true}
      // store={ }
      title="进销存系统"
    >
      <Route path="/error/:status" element={<PageError />} />
      <RoutesWithoutLayout>
        <Route path="/" element={<div>sss</div>} />
      </RoutesWithoutLayout>
    </CoreAdmin>
  )
}
