import {
  AuthProvider,
  UserIdentity,
  WithRedirectTo
} from "@rakit/core";
import { FetchFunction } from "@rakit/fetch";

async function login(
  fetch: FetchFunction,
  { username, password }: any,
): Promise<WithRedirectTo | void | any> {
  try {
    const auth = await fetch({
      url: "/login",
      method: "POST",
      body: { username, password },
    });
    localStorage.setItem('auth', JSON.stringify(auth));
  } catch {
    throw new Error('Network error');
  }
}

function logout(): Promise<void | false | string> {
  localStorage.removeItem('auth');
  return Promise.resolve();
}

function checkAuth(): Promise<void> {
  // // case 1:
  // return localStorage.getItem('auth')
  //   ? Promise.resolve()
  //   : Promise.reject({ message: 'login.required' });
  //
  // // case 2:
  return localStorage.getItem('auth')
    ? Promise.resolve()
    : Promise.reject({ redirectTo: '/no-access' })
}

function checkError(error: any): Promise<void> {
  const status = error.status;
    if (status === 401 || status === 403) {
      // // case 1:
      // localStorage.removeItem('auth');
      // return Promise.reject();
      //
      // // case 2:
      // localStorage.removeItem('auth');
      // return Promise.reject({ redirectTo: '/credentials-required' });
      //
      // case 3:
      return Promise.reject({
        redirectTo: '/unauthorized',
        logoutUser: false,
        message: 'Unauthorized user!', // or message: false
      });
    }
    // other error code (404, 500, etc): no need to log out
    return Promise.resolve();
}

function getIdentity(): Promise<UserIdentity> {
  try {
    const { id, fullName, avatar, phoneNumber, email } = JSON.parse(localStorage.getItem('auth')!);
    return Promise.resolve({
      id,
      fullName,
      avatar,
      phoneNumber,
      email,
    });
  } catch (error) {
    return Promise.reject(error);
  }
}

function getPermissions(): Promise<any> {
  return Promise.resolve('');
}

// async function handleCallback(): Promise<AuthRedirectResult | void | any> {
//   if (!query.includes('code=') && !query.includes('state=')) {
//     throw new Error('Failed to handle login callback.');
//   }
//   // If we did receive the Auth0 parameters,
//   // get an access token based on the query paramaters
//   await Auth0Client.handleRedirectCallback();
//   return { redirectTo: '/posts' };
// }

export function createAuthProvider(fetch: FetchFunction<any>): AuthProvider {
  return {
    login: (params) => login(fetch, params),
    logout,
    checkAuth,
    checkError,
    getIdentity,
    getPermissions,
    // handleCallback,
  };

}
