import Polyglot from 'node-polyglot';
import { I18nProvider, Locale } from "@rakit/core";
import merge from 'lodash/merge';
import { enUS, zhCN } from '@rakit/joy-ui';

const locale: Locale = { code: 'zh', name: "简体中文" };

const polyglot = new Polyglot({
    locale: locale.code,
    phrases: merge({
      '': '',
    }, enUS, zhCN),
});

function translate(key: string, options: any = {}) {
  if (polyglot.has(key)) {
    return polyglot.t(key, options);
  }
  return key;
}

export const i18nProvider: I18nProvider = {
  translate,
  changeLocale: () => Promise.resolve(),
  getLocale: () => "zh",
  getLocales: () => [locale],
}
