import { Fragment } from 'react';
import Box from '@mui/joy/Box';
import IconButton from '@mui/joy/IconButton';
import Typography from '@mui/joy/Typography';
import BadgeRoundedIcon from '@mui/icons-material/BadgeRounded';
import { ColorSchemeToggle, useMediaQuery } from '@rakit/joy-ui';
import { SignInCard } from 'src/views/SignInCard';
import { useDefaultTitle } from '@rakit/core';

export default function SignIn() {
  const isSmall = useMediaQuery((theme) => theme.breakpoints.down('md'));
  const title = useDefaultTitle();
  return (
    <Fragment>
      <Box
        sx={{
          display: 'flex',
          flexDirection: 'column',
          minHeight: '100dvh',
          width: '100%',
          px: 2,
          justifyContent: "space-between",
          background: isSmall ? "var(--palette-background-surface)" : undefined,
        }}
      >
        <Box
          component="header"
          sx={{
            py: 3,
            display: 'flex',
            justifyContent: 'space-between',
          }}
        >
          <Box sx={{ gap: 2, display: 'flex', alignItems: 'center' }}>
            <IconButton variant="soft" color="primary" size="sm">
              <BadgeRoundedIcon />
            </IconButton>
            <Typography level="title-lg">{title}</Typography>
          </Box>
          <ColorSchemeToggle variant={isSmall ? 'outlined' : 'plain'} />
        </Box>
        <SignInCard />
        <Box component="footer" sx={{ py: 3 }}>
          <Typography level="body-xs" sx={{ textAlign: 'center' }}>
            © Your company {new Date().getFullYear()}
          </Typography>
        </Box>
      </Box>
    </Fragment>
  );
}
