import { createContext } from "react";
import {
  BodyParams,
  Combination,
  ErrorCorrector,
  FetchConfig,
  Fetcher,
  RequestFactory,
  RequestIntercepter,
  ResponseTransformer,
  SearchParams
} from "./types";

export interface FormatOptions {
  bool?: (value: boolean) => string; // default to 'true' or 'false'
  null?: string; // default to '' or null
  nan?: string; // default to 'NaN'
  date?: (date: Date) => string; // default to Date.toISOString()
  skipNull?: boolean; // default to true
  skipEmptyString?: boolean; // default to true
  skipNaN?: boolean; // default to true
  arrayFormat?:
  | 'default'       // 'foo=1&foo=2&foo=3'
  | 'bracket'       // 'foo[]=1&foo[]=2&foo[]=3'
  | 'index'         // 'foo[0]=1&foo[1]=2&foo[3]=3'
  | 'dot'           // 'foo.0=1&foo.1=2&foo.2=3'
  | 'comma'         // 'foo=1,2,3'
  | 'line'          // 'foo=1|2|3'
  | 'bracket-comma' // 'foo[]=1,2,3'
  | 'bracket-line'; // 'foo[]=1|2|3'
  objectFormat?:
  | 'bracket' // 'foo[bar]=bazz'
  | 'dot'     // 'foo.bar=bazz'
  | 'colon';  // 'foo:bar=bazz'
}

export interface BodyMergeConfig extends FormatOptions {
  handle?: (
    bodys: BodyParams[],
    hasFiles: boolean | undefined,
    options: FormatOptions & { combination: Combination },
  ) => BodyParams;
}

export type BodyBuilder = (
  body: BodyParams,
  hasFiles: boolean,
  options: FormatOptions & { combination: Combination },
) => BodyInit | null;

export interface SearchParamsMergeConfig extends FormatOptions {
  handleIntoUrl?: (
    url: URL,
    params: SearchParams,
    options: FormatOptions & { combination: Combination },
  ) => void;
}

export interface FetchContextValue {
  baseUrl?: string;
  body?: BodyParams;
  bodyCombination?: Combination;
  bodyBuilder?: BodyBuilder;
  bodyMergeConfig?: BodyMergeConfig;
  config?: FetchConfig;
  configCombination?: Combination;
  correct?: ErrorCorrector;
  correctCombination?: Combination;
  createRequest?: RequestFactory;
  fetcher?: Fetcher;
  headers?: HeadersInit;
  headersCombination?: Combination;
  hasFiles?: boolean | undefined;
  intercept?: RequestIntercepter;
  interceptCombination?: Combination;
  searchParams?: SearchParams;
  searchParamsCombination?: Combination;
  searchParamsMergeConfig?: SearchParamsMergeConfig;
  transform?: ResponseTransformer;
  transformCombination?: Combination;
}

export const FetchContext = createContext<FetchContextValue | undefined>(undefined);
