import { useMemo } from "react";
import { FetchContext, FetchContextValue } from "./FetchContext";
import { useFetchContext } from "./useFetchContext";

export interface FetchContextProviderProps extends FetchContextValue {
  inherit?: boolean;
  children?: React.ReactNode;
}

export function FetchContextProvider(props: FetchContextProviderProps) {
  const {
    children,
    inherit,
    ...localContext
  } = props;

  const parentContext = useFetchContext();

  const value = useMemo<FetchContextValue>(() => {
    if (inherit) {
      return {
        ...parentContext,
        ...localContext,
      };
    }
    return {
      ...localContext,
    };
  }, [
    inherit,
    localContext,
    parentContext,
  ]);

  return (
    <FetchContext.Provider value={value}>
      {children}
    </FetchContext.Provider>
  )
}
