import { useCallback } from "react";
import { BodyParams, Combination } from "./types";
import { useFetchContext } from "./useFetchContext";
import { buildBody, mergeBodys } from "./utils";

export interface CreateBodyOptions {
  body?: BodyParams;
  bodyCombination?: Combination;
  hasFiles?: boolean;
  [key: string]: any;
}

export type CreateBodyFunction = (
  body?: BodyParams | undefined,
  hasFiles?: boolean | undefined,
) => BodyInit | null;

export function useCreateBody(options?: CreateBodyOptions) {
  const fetchContext = useFetchContext();

  return useCallback((newOptions?: CreateBodyOptions): BodyInit | null => {
    const bodyBuilder = fetchContext?.bodyBuilder ?? buildBody;
    const handle = fetchContext?.bodyMergeConfig?.handle ?? mergeBodys;
    const mergeConfig = fetchContext?.bodyMergeConfig ?? {};
    const combination = newOptions?.bodyCombination
      ?? options?.bodyCombination
      ?? fetchContext?.bodyCombination
      ?? 'chaining';
    const bodys = toList(
      fetchContext?.body,
      options?.body,
      newOptions?.body,
    );
    const hasFiles = newOptions?.hasFiles === true
      || options?.hasFiles === true
      || fetchContext?.hasFiles === true;
    const handleOptions = { ...mergeConfig, combination }
    const body = handle(bodys, hasFiles, handleOptions);
    return bodyBuilder(body, hasFiles, handleOptions);
  }, [
    fetchContext?.body,
    fetchContext?.bodyBuilder,
    fetchContext?.bodyCombination,
    fetchContext?.bodyMergeConfig,
    fetchContext?.hasFiles,
    options?.body,
    options?.bodyCombination,
    options?.hasFiles,
  ]);
}

const toList = <T>(...items: Array<T | undefined>): T[] => {
  return items.filter(i => i !== undefined);
}
