import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";
import { Combination, ErrorCorrector } from "./types";

export interface CreateErrorCorrectorOptions {
  correct?: ErrorCorrector;
  correctCombination?: Combination;
  [key: string]: any;
}

export type CreateErrorCorrectorFunction = (
  options?: CreateErrorCorrectorOptions
) => ErrorCorrector;

export function useCreateErrorCorrector(
  options?: CreateErrorCorrectorOptions,
): CreateErrorCorrectorFunction {
  const fetchContext = useFetchContext();

  return useCallback((newOptions?: CreateErrorCorrectorOptions): ErrorCorrector => {
    return (
      error: unknown,
      request: Request | undefined,
      response: Response | undefined
    ): Promise<any> => {
      const combination = newOptions?.correctCombination
        ?? options?.correctCombination
        ?? fetchContext?.correctCombination
        ?? 'overrides';
      if (combination === "chaining") {
        return [
          fetchContext?.correct,
          options?.correct,
          newOptions?.correct,
        ].reduce<Promise<any>>(
          async (promise, handle) => {
            if (!handle) {
              return promise;
            } else {
              return promise.catch(error => handle(error, request, response))
            }
          },
          defaultCorrect(error)
        );
      }
      const handle = newOptions?.correct
        ?? options?.correct
        ?? fetchContext?.correct
        ?? defaultCorrect;
      return handle(error, request, response);
    }
  }, [
    fetchContext?.correct,
    fetchContext?.correctCombination,
    options?.correct,
    options?.correctCombination,
  ]);
}

function defaultCorrect(error: unknown): Promise<any> {
  return Promise.reject(error);
}
