import { useCallback } from "react";
import { Combination, FetchConfig } from "./types";
import { useFetchContext } from "./useFetchContext";

export interface CreateFetchConfigOptions {
  config?: FetchConfig;
  configCombination?: Combination;
  [key: string]: any;
}

export type CreateFetchConfigFunction = (options?: CreateFetchConfigOptions) => FetchConfig;

export function useCreateFetchConfig(
  options?: CreateFetchConfigOptions,
): CreateFetchConfigFunction {
  const fetchContext = useFetchContext({});

  return useCallback((newOptions?: CreateFetchConfigOptions): FetchConfig => {
    const combination = newOptions?.configCombination
      ?? options?.configCombination
      ?? fetchContext?.configCombination
      ?? 'chaining';
    if (combination === 'chaining') {
      return {
        ...fetchContext?.config,
        ...options?.config,
        ...newOptions?.config,
      }
    }
    return newOptions?.config
      ?? options?.config
      ?? fetchContext?.config
      ?? {};
  }, [
    fetchContext?.config,
    fetchContext?.configCombination,
    options?.config,
    options?.configCombination,
  ]);
}
