import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";
import { Fetcher } from "./types";

export interface CreateFetcherOptions {
  fetcher?: Fetcher;
  [key: string]: any;
}

export const useCreateFetcher = (options?: CreateFetcherOptions) => {
  const fetchContext = useFetchContext({});

  return useCallback((newOptions?: CreateFetcherOptions) => {
    return (input: RequestInfo | URL, init?: RequestInit) => {
      const handle = newOptions?.fetcher
        ?? options?.fetcher
        ?? fetchContext?.fetcher
        ?? globalThis.fetch;
      return handle(input, init);
    }
  }, [
    fetchContext?.fetcher,
    options?.fetcher
  ]);
}
