import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";
import { Combination } from "./types";

export interface CreateHeadersOptions {
  headers?: HeadersInit;
  headersCombination?: Combination;
  [key: string]: any;
}

export type CreateHeadersFunction = (
  options?: CreateHeadersOptions
) => HeadersInit | undefined;

export function useCreateHeaders(options?: CreateHeadersOptions): CreateHeadersFunction {
  const fetchContext = useFetchContext();

  return useCallback((newOptions?: CreateHeadersOptions): HeadersInit | undefined => {
    const combination = newOptions?.headersCombination
      ?? options?.headersCombination
      ?? fetchContext?.headersCombination
      ?? 'chaining';
    if (combination === 'chaining') {
      return [
        fetchContext?.headers,
        options?.headers,
        newOptions?.headers,
      ].reduce((acc, item) => {
        if (!item) {
          return acc;
        }
        const headers = new Headers(item);
        if (!acc) {
          return headers;
        }
        headers.forEach((value, name) => {
          (acc as Headers).set(name, value);
        });
        return acc;
      });
    }
    return newOptions?.headers
      ?? options?.headers
      ?? fetchContext?.headers;
  }, [
    fetchContext?.headers,
    fetchContext?.headersCombination,
    options?.headers,
    options?.headersCombination,
  ]);
}
