import { useCallback } from "react";
import { FetchConfig, RequestFactory } from "./types";
import { useFetchContext } from "./useFetchContext";

export interface UseCreateRequestOptions {
  createRequest?: RequestFactory;
  [key: string]: any;
}

export type CreateRequestFunction = (
  url: URL,
  config: FetchConfig,
  headers: HeadersInit | undefined,
  body: BodyInit | null | undefined,
  signal?: AbortSignal,
) => Promise<Request> | Request;

const defaultCreateRequest = (
  input: RequestInfo | URL,
  init?: RequestInit
) => {
  return new Request(input, init);
};

export function useCreateRequest(options?: UseCreateRequestOptions) {
  const fetchContext = useFetchContext();

  return useCallback(async (
    url: URL,
    config: FetchConfig,
    headers: HeadersInit | undefined,
    body: BodyInit | null | undefined,
    signal?: AbortSignal,
  ): Promise<Request> => {
    const handle = options?.createRequest
      ?? fetchContext?.createRequest
      ?? defaultCreateRequest;
    return await handle(url, {
      ...config,
      headers,
      body,
      signal,
    });
  }, [
    fetchContext?.createRequest,
    options?.createRequest,
  ]);
}
