import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";
import { Combination, RequestIntercepter } from "./types";

export interface CreateRequestIntercepterOptions {
  intercept?: RequestIntercepter;
  interceptCombination?: Combination;
  [key: string]: any;
}

const defaultIntercept = (request: Request) => Promise.resolve(request);

export function useCreateRequestIntercepter(
  options?: CreateRequestIntercepterOptions
) {
  const fetchContext = useFetchContext();
  return useCallback((newOptions?: CreateRequestIntercepterOptions) => {
    return (request: Request) => {
      const combination = newOptions?.interceptCombination
        ?? options?.interceptCombination
        ?? fetchContext?.interceptCombination
        ?? 'overrides';
      if (combination === 'chaining') {
        return [
          fetchContext?.intercept,
          options?.intercept,
          newOptions?.intercept,
        ].reduce<Promise<Request>>(
          (promise, handle) => {
            if (!handle) {
              return promise;
            } else {
              return promise.then(async request => {
                return await handle(request);
              });
            }
          },
          defaultIntercept(request),
        );
      }
      const handle = newOptions?.intercept
        ?? options?.intercept
        ?? fetchContext?.intercept
        ?? defaultIntercept
      return handle(request);
    }
  }, [
    options?.intercept,
    fetchContext?.intercept,
  ]);
}
