import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";
import { Combination, ResponseTransformer } from "./types";

function defaultTransform(_: Request, response: Response) {
  return response.clone().json();
}

export interface CreateResponseTransformerOptions {
  transform?: ResponseTransformer;
  transformCombination?: Combination;
  [key: string]: any;
}

export function useCreateResponseTransformer(options?: CreateResponseTransformerOptions) {
  const fetchContext = useFetchContext();
  return useCallback((newOptions?: CreateResponseTransformerOptions): ResponseTransformer => {
    return (request: Request, response: Response) => {
      const combination = newOptions?.transformCombination
        ?? options?.transformCombination
        ?? fetchContext?.transformCombination
        ?? 'overrides';
      if (combination === 'chaining') {
        return [
          fetchContext?.transform,
          options?.transform,
          newOptions?.transform,
        ].reduce<Promise<any>>(
          (promise, handle) => {
            if (!handle) {
              return promise;
            } else {
              return promise.then(async data => {
                return await handle(request, response, data);
              });
            }
          },
          defaultTransform(request, response),
        );
      }
      const handle = newOptions?.transform
        ?? options?.transform
        ?? fetchContext?.transform
        ?? defaultTransform;
      return handle(request, response);
    };
  }, [
    fetchContext?.transform,
    fetchContext?.transformCombination,
    options?.transform,
    options?.transformCombination,
  ]);
}
