import { useCallback } from "react";
import { Combination, SearchParams } from "./types";
import { useFetchContext } from "./useFetchContext";
import { searchParamsIntoUrl } from "./utils";

export interface CreateUrlOptions {
  baseUrl?: string;
  searchParams?: SearchParams;
  searchParamsCombination?: Combination;
  [key: string]: any;
}

export type CreateUrlFunction = (
  newOptions: CreateUrlOptions & { url: string }
) => URL;

export function useCreateUrl(options?: CreateUrlOptions): CreateUrlFunction {
  const fetchContext = useFetchContext();

  return useCallback((newOptions: CreateUrlOptions & { url: string }): URL => {
    const mergeConfig = fetchContext?.searchParamsMergeConfig ?? {};
    const handleIntoUrl = mergeConfig.handleIntoUrl ?? searchParamsIntoUrl;
    const baseUrl = newOptions.baseUrl
      ?? options?.baseUrl
      ?? fetchContext?.baseUrl;
    const uri = new URL(newOptions.url, baseUrl);
    const combination = newOptions.searchParamsCombination
      ?? options?.searchParamsCombination
      ?? fetchContext?.searchParamsCombination
      ?? 'chaining';
    const handleOptions = {
      ...mergeConfig,
      combination,
    };

    if (combination === 'overrides') {
      const searchParams = newOptions.searchParams
        ?? options?.searchParams
        ?? fetchContext?.searchParams;
      if (searchParams != null) {
        handleIntoUrl(uri, searchParams, handleOptions);
      }
      return uri;
    }

    if (fetchContext?.searchParams != null) {
      handleIntoUrl(uri, fetchContext.searchParams, handleOptions);
    }
    if (options?.searchParams != null) {
      handleIntoUrl(uri, options.searchParams, handleOptions);
    }
    if (newOptions.searchParams != null) {
      handleIntoUrl(uri, newOptions.searchParams, handleOptions);
    }
    return uri;
  }, [
    fetchContext?.baseUrl,
    fetchContext?.searchParams,
    fetchContext?.searchParamsCombination,
    fetchContext?.searchParamsMergeConfig,
    options?.baseUrl,
    options?.searchParams,
    options?.searchParamsCombination,
  ]);
}
