import Sheet from '@mui/joy/Sheet';
import Typography from '@mui/joy/Typography';
import FormControl from '@mui/joy/FormControl';
import FormLabel from '@mui/joy/FormLabel';
import Input from '@mui/joy/Input';
import Button from '@mui/joy/Button';
import Link from '@mui/joy/Link';
import Box from '@mui/joy/Box';
import Divider from '@mui/joy/Divider';
import Stack from '@mui/joy/Stack';
import Checkbox from '@mui/joy/Checkbox';
import { FormEvent } from "react";

interface FormElements extends HTMLFormControlsCollection {
  email: HTMLInputElement;
  password: HTMLInputElement;
  persistent: HTMLInputElement;
}

interface SignInFormElement extends HTMLFormElement {
  readonly elements: FormElements;
}

export function LoginForm() {
  return (
    <form
      onSubmit={(event: FormEvent<SignInFormElement>) => {
        event.preventDefault();
        const formElements = event.currentTarget.elements;
        const data = {
          email: formElements.email.value,
          password: formElements.password.value,
          persistent: formElements.persistent.checked,
        };
        alert(JSON.stringify(data, null, 2));
      }}
    >
      <FormControl required>
        <FormLabel>您的账户</FormLabel>
        <Input
          type="text"
          name="account"
          placeholder="用户名/手机/邮箱"
          size="lg"
          sx={{ fontSize: "md" }}
        />
      </FormControl>
      <FormControl required>
        <FormLabel>登录密码</FormLabel>
        <Input
          type="password"
          name="password"
          placeholder="至少6位"
          size="lg"
          sx={{ fontSize: "md" }}
        />
      </FormControl>
      <Stack sx={{ gap: 4, mt: 2 }}>
        <Box
          sx={{
            display: 'flex',
            justifyContent: 'space-between',
            alignItems: 'center',
          }}
        >
          <Checkbox size="sm" label="记住我" name="persistent" />
          <Link level="title-sm" href="#replace-with-a-link">
            忘记了密码？
          </Link>
        </Box>
        <Button type="submit" fullWidth size="lg">
          登录
        </Button>
      </Stack>
    </form>
  )
}
