import {
  Portlet,
  useDefaultTitle,
  useRecordRepresentation,
  useTranslate
} from "@rakit/core";
import { ReactElement, JSX } from "react";

export interface PageTitleProps extends JSX.ElementAttributesProperty {
  record?: any;
  preferenceKey?: string | false;
  title?: string | ReactElement;
}

export function PageTitle(props: PageTitleProps) {
  const {
    record,
    preferenceKey,
    title,
    ...rest
  } = props;

  const translate = useTranslate();
  const defaultTitle = useDefaultTitle();
  const titleFromPreferences = useRecordRepresentation({
    record,
    representation: preferenceKey === false ? undefined : preferenceKey,
  });

  if (!title && !titleFromPreferences && !defaultTitle) {
    return null;
  }

  return (
    <Portlet to="page-title">
      <span {...rest}>
        {
          titleFromPreferences
            ? translate(
              titleFromPreferences,
              { ...record, _: titleFromPreferences },
            )
            : !title
              ? defaultTitle
              : typeof title === "string"
                ? translate(title, { _: title })
                : title
        }
      </span>
    </Portlet>
  );
}
