import History from '@mui/icons-material/History';
import ErrorIcon from '@mui/icons-material/Report';
import Accordion from '@mui/joy/Accordion';
import AccordionDetails from '@mui/joy/AccordionDetails';
import AccordionSummary from '@mui/joy/AccordionSummary';
import Box from '@mui/joy/Box';
import Button from '@mui/joy/Button';
import { styled } from '@mui/joy/styles';
import Typography from '@mui/joy/Typography';
import {
  useDefaultTitle,
  useErrorContext,
  useResetErrorBoundaryOnLocationChange,
  useTranslate
} from '@rakit/core';
import { ColorSchemeToggle } from './ColorSchemeToggle';

export function RuntimeError() {
  const title = useDefaultTitle();
  const { error, errorInfo, resetErrorBoundary } = useErrorContext();
  const translate = useTranslate();

  useResetErrorBoundaryOnLocationChange(resetErrorBoundary);

  return (
    <Box
      sx={{
        display: "flex",
        flexDirection: "column",
        justifyContent: {
          sm: "flex-start",
          md: "center",
        },
        alignItems: "center",
        pt: 12,
        pb: 8,
        minHeight: "100dvh",
      }}
    >
      <Box sx={{
        position: "fixed",
        top: 0,
        insetInline: 0,
        display: "flex",
        justifyContent: "space-between",
        p: 4,
      }}>
        <div>{title}</div>
        <ColorSchemeToggle variant="plain" />
      </Box>
      <Root>
        <h1 className={ErrorClasses.title} role="alert">
          <ErrorIcon className={ErrorClasses.icon} />
          {translate('ra.page.error')}
        </h1>
        <div>{translate('ra.message.error')}</div>
        {process.env.NODE_ENV !== 'production' && (
          <>
            <Accordion className={ErrorClasses.panel}>
              <AccordionSummary className={ErrorClasses.panelSumary}>
                {translate(error.message)}
              </AccordionSummary>
              <AccordionDetails className={ErrorClasses.panelDetails}>
                {/* error message is repeated here to allow users to copy it. AccordionSummary doesn't support text selection. */}
                <p>{translate(error.message)}</p>
                <p>{errorInfo?.componentStack}</p>
              </AccordionDetails>
            </Accordion>

            <div className={ErrorClasses.advice}>
              <Typography textAlign="center">
                Need help with this error? Try the following:
              </Typography>
              <Typography component="div">
                <ul>
                  <li>
                    Check the{' '}
                    <a href="https://marmelab.com/react-admin/documentation.html">
                      react-admin documentation
                    </a>
                  </li>
                  <li>
                    Search on{' '}
                    <a href="https://stackoverflow.com/questions/tagged/react-admin">
                      StackOverflow
                    </a>{' '}
                    for community answers
                  </li>
                  <li>
                    Get help from the core team via{' '}
                    <a href="https://react-admin-ee.marmelab.com/#fromsww">
                      react-admin Enterprise Edition
                    </a>
                  </li>
                </ul>
              </Typography>
            </div>
          </>
        )}
        <div className={ErrorClasses.toolbar}>
          <Button
            variant="solid"
            startDecorator={<History />}
            onClick={goBack}
          >
            {translate('ra.action.back')}
          </Button>
        </div>
      </Root>
    </Box>
  );
}

const PREFIX = 'RaError';

export const ErrorClasses = {
  container: `${PREFIX}-container`,
  title: `${PREFIX}-title`,
  icon: `${PREFIX}-icon`,
  panel: `${PREFIX}-panel`,
  panelSumary: `${PREFIX}-panelSumary`,
  panelDetails: `${PREFIX}-panelDetails`,
  toolbar: `${PREFIX}-toolbar`,
  advice: `${PREFIX}-advice`,
};

const Root = styled('div', {
  name: PREFIX,
  overridesResolver: (_, styles) => styles.root,
})(({ theme }) => ({
  display: 'flex',
  flexDirection: 'column',
  alignItems: 'center',
  justifyContent: 'center',
  [theme.breakpoints.down('md')]: {
    padding: '1em',
  },
  fontFamily: 'Roboto, sans-serif',
  opacity: 0.5,

  [`& .${ErrorClasses.title}`]: {
    display: 'flex',
    alignItems: 'center',
  },

  [`& .${ErrorClasses.icon}`]: {
    width: '2em',
    height: '2em',
    marginRight: '0.5em',
  },

  [`& .${ErrorClasses.panel}`]: {
    marginTop: '1em',
    maxWidth: '60em',
  },

  [`& .${ErrorClasses.panelSumary}`]: {
    userSelect: 'all',
  },

  [`& .${ErrorClasses.panelDetails}`]: {
    whiteSpace: 'pre-wrap',
  },

  [`& .${ErrorClasses.toolbar}`]: {
    marginTop: '2em',
  },

  [`& .${ErrorClasses.advice}`]: {
    marginTop: '2em',
  },
}));

function goBack() {
  window.history.go(-1);
}
