import Box from '@mui/joy/Box';
import Drawer from '@mui/joy/Drawer';
import Button from '@mui/joy/Button';
import Card from '@mui/joy/Card';
import CardContent from '@mui/joy/CardContent';
import Checkbox from '@mui/joy/Checkbox';
import DialogTitle from '@mui/joy/DialogTitle';
import DialogContent from '@mui/joy/DialogContent';
import ModalClose from '@mui/joy/ModalClose';
import Divider from '@mui/joy/Divider';
import FormControl from '@mui/joy/FormControl';
import FormLabel from '@mui/joy/FormLabel';
import List from '@mui/joy/List';
import ListItem from '@mui/joy/ListItem';
import ListItemDecorator from '@mui/joy/ListItemDecorator';
import Stack from '@mui/joy/Stack';
import RadioGroup from '@mui/joy/RadioGroup';
import Radio from '@mui/joy/Radio';
import Sheet from '@mui/joy/Sheet';
import Typography from '@mui/joy/Typography';
import TuneIcon from '@mui/icons-material/TuneRounded';
import HomeRoundedIcon from '@mui/icons-material/HomeRounded';
import ApartmentRoundedIcon from '@mui/icons-material/ApartmentRounded';
import MeetingRoomRoundedIcon from '@mui/icons-material/MeetingRoomRounded';
// import HotelRoundedIcon from '@mui/icons-material/HotelRounded';
import Person from '@mui/icons-material/Person';
import People from '@mui/icons-material/People';
// import Apartment from '@mui/icons-material/Apartment';
// import Done from '@mui/icons-material/Done';
import { Fragment, useState } from 'react';
import IcContrastSvg from "../assets/ic-contrast.svg";
// import IcAutofitWidthSvg from "../assets/ic-autofit-width.svg";
import IcSidebarOutlineSvg from "../assets/ic-sidebar-outline.svg";
import IcSidebarFilledSvg from "../assets/ic-sidebar-filled.svg";

const sidebarSettings = [
  {
    id: "integrate",
    label: "Integrate",
    icon: IcSidebarOutlineSvg,
  },
  {
    id: "apparent",
    label: "Apparent",
    icon: IcSidebarFilledSvg,
  },
]

const genericSettings = [
  {
    id: "comtrast",
    label: "Comtrast",
    icon: IcContrastSvg,
  },
  {
    id: "compact",
    label: "Compact",
    icon: IcSidebarFilledSvg,
  },
]

export default function Settings() {
  const [open, setOpen] = useState(false);
  const [type, setType] = useState('Guesthouse');
  const [, setAmenities] = useState([0, 6]);

  return (
    <Fragment>
      <Button
        variant="outlined"
        color="neutral"
        startDecorator={<TuneIcon />}
        onClick={() => setOpen(true)}
      >
        Change filters
      </Button>

      <Drawer
        size="md"
        variant="plain"
        open={open}
        onClose={() => setOpen(false)}
        slotProps={{
          content: {
            sx: {
              bgcolor: 'transparent',
              p: { md: 3, sm: 0 },
              boxShadow: 'none',
            },
          },
        }}
      >
        <Sheet
          sx={{
            borderRadius: 'md',
            p: 2,
            display: 'flex',
            flexDirection: 'column',
            gap: 2,
            height: '100%',
            overflow: 'auto',
          }}
        >
          <DialogTitle>Filters</DialogTitle>
          <ModalClose />
          <Divider sx={{ mt: 'auto' }} />

          <DialogContent sx={{ gap: 2 }}>
            <FormControl>
              <FormLabel sx={{ typography: 'title-md', fontWeight: 'bold' }}>
                Property type
              </FormLabel>
              <RadioGroup
                value={type || ''}
                onChange={(event) => {
                  setType(event.target.value);
                }}
              >
                <Box
                  sx={{
                    display: 'grid',
                    gridTemplateColumns: 'repeat(auto-fill, minmax(140px, 1fr))',
                    gap: 1.5,
                  }}
                >
                  {[
                    {
                      name: '浅色',
                      icon: <HomeRoundedIcon />,
                    },
                    {
                      name: '深色',
                      icon: <ApartmentRoundedIcon />,
                    },
                    {
                      name: '自动',
                      icon: <MeetingRoomRoundedIcon />,
                    },
                  ].map((item) => (
                    <Card
                      key={item.name}
                      sx={{
                        boxShadow: 'none',
                        '&:hover': { bgcolor: 'background.level1' },
                      }}
                    >
                      <CardContent>
                        {item.icon}
                        <Typography level="title-md">{item.name}</Typography>
                      </CardContent>
                      <Radio
                        disableIcon
                        overlay
                        checked={type === item.name}
                        variant="outlined"
                        color="neutral"
                        value={item.name}
                        sx={{ mt: -2 }}
                        slotProps={{
                          action: {
                            sx: {
                              ...(type === item.name && {
                                borderWidth: 2,
                                borderColor:
                                  'var(--joy-palette-primary-outlinedBorder)',
                              }),
                              '&:hover': {
                                bgcolor: 'transparent',
                              },
                            },
                          },
                        }}
                      />
                    </Card>
                  ))}
                </Box>
              </RadioGroup>
            </FormControl>

            <Typography level="title-md" sx={{ fontWeight: 'bold', mt: 1 }}>
              侧边栏
            </Typography>
            <RadioGroup aria-label="Your plan" name="people" defaultValue="Individual">
              <List
                sx={{
                  minWidth: 240,
                  '--List-gap': '0.5rem',
                  '--ListItem-paddingY': '1rem',
                  '--ListItem-radius': '8px',
                  '--ListItemDecorator-size': '32px',
                }}
              >
                {sidebarSettings.map((item, index) => (
                  <ListItem variant="outlined" key={item.id} sx={{ boxShadow: 'sm' }}>
                    <ListItemDecorator>
                      {[<Person key="1" />, <People key="2" />][index]}
                    </ListItemDecorator>
                    <Radio
                      overlay
                      value={item.id}
                      label={item.label}
                      sx={{ flexGrow: 1, flexDirection: 'row-reverse' }}
                      slotProps={{
                        action: ({ checked }) => ({
                          sx: (theme) => ({
                            ...(checked && {
                              inset: -1,
                              border: '2px solid',
                              borderColor: theme.vars.palette.primary[500],
                            }),
                          }),
                        }),
                      }}
                    />
                  </ListItem>
                ))}
              </List>
            </RadioGroup>

            <Typography id="sandwich-group" level="title-md" sx={{ fontWeight: 'bold', mt: 1 }}>
              Settings
            </Typography>
            <div role="group" aria-labelledby="sandwich-group">
              <List
                sx={{
                  minWidth: 240,
                  '--List-gap': '0.5rem',
                  '--ListItem-paddingY': '1rem',
                  '--ListItem-radius': '8px',
                  '--ListItemDecorator-size': '32px',
                }}
              >
                {genericSettings.map((item, index) => (
                  <ListItem variant="outlined" key={item.id} sx={{ boxShadow: 'sm' }}>
                    <ListItemDecorator>
                      {[<Person key="p" />, <People key="2" />][index]}
                    </ListItemDecorator>
                    <Checkbox
                      overlay
                      value={item.id}
                      label={item.label}
                      sx={{ flexGrow: 1, flexDirection: 'row-reverse' }}
                      slotProps={{
                        action: ({ checked }) => ({
                          sx: (theme) => ({
                            ...(checked && {
                              inset: -1,
                              border: '2px solid',
                              borderColor: theme.vars.palette.primary[500],
                            }),
                          }),
                        }),
                      }}
                    />
                  </ListItem>
                ))}
              </List>
            </div>

            <Typography level="title-md" sx={{ fontWeight: 'bold', mt: 1 }}>
              Presets
            </Typography>
            <FormControl>
              <FormLabel sx={{ typography: 'title-md', fontWeight: 'bold' }}>
                Presets
              </FormLabel>
              <RadioGroup
                value={type || ''}
                onChange={(event) => {
                  setType(event.target.value);
                }}
              >
                <Box
                  sx={{
                    display: 'grid',
                    gridTemplateColumns: 'repeat(auto-fill, minmax(140px, 1fr))',
                    gap: 1.5,
                  }}
                >
                  {[
                    {
                      name: '浅色',
                      icon: <HomeRoundedIcon />,
                    },
                    {
                      name: '深色',
                      icon: <ApartmentRoundedIcon />,
                    },
                    {
                      name: '自动',
                      icon: <MeetingRoomRoundedIcon />,
                    },
                  ].map((item) => (
                    <Card
                      key={item.name}
                      sx={{
                        boxShadow: 'none',
                        '&:hover': { bgcolor: 'background.level1' },
                      }}
                    >
                      <CardContent>
                        {item.icon}
                        <Typography level="title-md">{item.name}</Typography>
                      </CardContent>
                      <Radio
                        disableIcon
                        overlay
                        checked={type === item.name}
                        variant="outlined"
                        color="neutral"
                        value={item.name}
                        sx={{ mt: -2 }}
                        slotProps={{
                          action: {
                            sx: {
                              ...(type === item.name && {
                                borderWidth: 2,
                                borderColor:
                                  'var(--joy-palette-primary-outlinedBorder)',
                              }),
                              '&:hover': {
                                bgcolor: 'transparent',
                              },
                            },
                          },
                        }}
                      />
                    </Card>
                  ))}
                </Box>
              </RadioGroup>
            </FormControl>
          </DialogContent>

          <Divider sx={{ mt: 'auto' }} />
          <Stack
            direction="row"
            useFlexGap
            spacing={1}
            sx={{ justifyContent: 'space-between' }}
          >
            <Button
              variant="outlined"
              color="neutral"
              onClick={() => {
                setType('');
                setAmenities([]);
              }}
            >
              Clear
            </Button>
            <Button onClick={() => setOpen(false)}>Show 165 properties</Button>
          </Stack>
        </Sheet>
      </Drawer>
    </Fragment>
  );
}
