import * as React from "react";
import GlobalStyles from "@mui/joy/GlobalStyles";
import Avatar from "@mui/joy/Avatar";
import Box from "@mui/joy/Box";
import Button from "@mui/joy/Button";
import Card from "@mui/joy/Card";
import Chip, { type ChipProps } from "@mui/joy/Chip";
import Divider from "@mui/joy/Divider";
import IconButton from "@mui/joy/IconButton";
// import Input from "@mui/joy/Input"
import LinearProgress from "@mui/joy/LinearProgress";
import List from "@mui/joy/List";
import ListItem from "@mui/joy/ListItem";
import ListItemButton, { listItemButtonClasses } from "@mui/joy/ListItemButton";
import ListItemContent from "@mui/joy/ListItemContent";
import Typography from "@mui/joy/Typography";
import Sheet from "@mui/joy/Sheet";
import Stack from "@mui/joy/Stack";
// import SearchRoundedIcon from "@mui/icons-material/SearchRounded"
import HomeRoundedIcon from "@mui/icons-material/HomeRounded";
import DashboardRoundedIcon from "@mui/icons-material/DashboardRounded";
import ShoppingCartRoundedIcon from "@mui/icons-material/ShoppingCartRounded";
import AssignmentRoundedIcon from "@mui/icons-material/AssignmentRounded";
import QuestionAnswerRoundedIcon from "@mui/icons-material/QuestionAnswerRounded";
import GroupRoundedIcon from "@mui/icons-material/GroupRounded";
import SupportRoundedIcon from "@mui/icons-material/SupportRounded";
import SettingsRoundedIcon from "@mui/icons-material/SettingsRounded";
import CloseRoundedIcon from "@mui/icons-material/CloseRounded";
import LogoutRoundedIcon from "@mui/icons-material/LogoutRounded";
import BrightnessAutoRoundedIcon from "@mui/icons-material/BrightnessAutoRounded";
import KeyboardArrowDownIcon from "@mui/icons-material/KeyboardArrowDown";

import { ColorSchemeToggle } from "./ColorSchemeToggle";
import { closeSidebar } from "./utils";

interface ChildItem {
  name: string
  label: string
  badge?: React.ComponentType<ChipProps>
}

interface RootItem extends ChildItem {
  icon: React.ComponentType
  label: string
  children?: ChildItem[]
}

const items: RootItem[] = [
  {
    name: "home",
    icon: HomeRoundedIcon,
    label: "Home",
  },
  {
    name: "dashboard",
    icon: DashboardRoundedIcon,
    label: "Dashboard",
  },
  {
    name: "orders",
    icon: ShoppingCartRoundedIcon,
    label: "Orders",
  },
  {
    name: "tasks",
    icon: AssignmentRoundedIcon,
    label: "Tasks",
    children: [
      { name: "all", label: "All tasks" },
      { name: "backlog", label: "Backlog" },
      { name: "inProgress", label: "In progress" },
      { name: "done", label: "Done" },
    ],
  },
  {
    name: "messages",
    icon: QuestionAnswerRoundedIcon,
    label: "Messages",
    badge: EventBadge,
  },
  {
    name: "users",
    icon: GroupRoundedIcon,
    label: "Users",
    children: [
      { name: "myProfile", label: "My profile" },
      { name: "create", label: "Create a new user" },
      { name: "roles&permissions", label: "Roles & permission" },
    ],
  },
  // {
  //   icon: SupportRoundedIcon,
  //   label: "Support",
  // },
  // {
  //   icon: SettingsRoundedIcon,
  //   label: "Settings",
  // }
];

function RenderRootItem(props: RootItem) {
  const {
    icon: Icon,
    badge: Badge,
    label,
    children,
  } = props;

  // TODO
  const selected = label === "Orders";

  if (!children?.length) {
    return (
      <ListItem>
        <ListItemButton
          role="menuitem"
          component="a"
          href="/joy-ui/getting-started/templates/messages/"
          selected={selected}
        >
          <Icon />
          <ListItemContent>
            <Typography level="title-sm">{label}</Typography>
          </ListItemContent>
          {Badge && <Badge size="sm" color="primary" variant="solid" />}
        </ListItemButton>
      </ListItem>
    );
  }

  return (
    <ListItem nested>
      <Toggler
        renderToggle={({ open, setOpen }) => (
          <ListItemButton
            selected={selected}
            onClick={() => setOpen(!open)}
          >
            <Icon />
            <ListItemContent>
              <Typography level="title-sm">{label}</Typography>
            </ListItemContent>
            {Badge && <Badge size="sm" color="primary" variant="solid" />}
            <KeyboardArrowDownIcon
              sx={{ transform: open ? "rotate(180deg)" : "none" }}
            />
          </ListItemButton>
        )}
      >
        <List sx={{ gap: 0.5 }}>
          {children.map(({ label, badge: Badge }, index) => (
            <ListItem
              key={label}
              sx={{ mt: index > 0 ? 0 : 0.5 }}
            >
              <ListItemButton
                role="menuitem"
                component="a"
                href="/joy-ui/getting-started/templates/profile-dashboard/"
                selected={selected}
              >
                <RingIcon />
                <ListItemContent>{label}</ListItemContent>
                {Badge && <Badge size="sm" color="primary" variant="solid" />}
              </ListItemButton>
            </ListItem>
          ))}
        </List>
      </Toggler>
    </ListItem>
  );
}

function EventBadge(props: ChipProps) {
  return (
    <Chip {...props}>4</Chip>
  );
}

function RingIcon() {
  return (
    <Box
      sx={{
        width: "8px",
        height: "8px",
        borderRadius: "4px",
        boxSizing: "border-box",
        border: "2px solid #ccc"
      }}
    />
  );
}

function Toggler({
  defaultExpanded = false,
  renderToggle,
  children,
}: {
  defaultExpanded?: boolean;
  children: React.ReactNode;
  renderToggle: (params: {
    open: boolean;
    setOpen: React.Dispatch<React.SetStateAction<boolean>>;
  }) => React.ReactNode;
}) {
  const [open, setOpen] = React.useState(defaultExpanded);
  return (
    <React.Fragment>
      {renderToggle({ open, setOpen })}
      <Box
        sx={{
          display: "grid",
          gridTemplateRows: open ? "1fr" : "0fr",
          transition: "0.2s ease",
          "& > *": {
            overflow: "hidden",
          },
        }}
      >
        {children}
      </Box>
    </React.Fragment>
  );
}

export function Sidebar() {
  return (
    <Sheet
      className="Sidebar"
      sx={{
        position: { xs: "fixed", md: "sticky" },
        transform: {
          xs: "translateX(calc(100% * (var(--SideNavigation-slideIn, 0) - 1)))",
          md: "none",
        },
        transition: "transform 0.4s, width 0.4s",
        zIndex: 10000,
        height: "100dvh",
        width: "var(--Sidebar-width)",
        top: 0,
        // p: 2,
        flexShrink: 0,
        display: "flex",
        flexDirection: "column",
        gap: 2,
        borderRight: "1px solid",
        borderColor: "divider",
      }}
    >
      <GlobalStyles
        styles={(theme) => ({
          ":root": {
            "--Sidebar-width": "80%",
            [theme.breakpoints.up("sm")]: {
              "--Sidebar-width": "40%",
            },
            [theme.breakpoints.up("md")]: {
              "--Sidebar-width": "240px",
            },
            [theme.breakpoints.up("lg")]: {
              "--Sidebar-width": "260px",
            },
          },
        })}
      />
      <Box
        className="Sidebar-overlay"
        sx={{
          position: "fixed",
          zIndex: 9998,
          top: 0,
          left: 0,
          width: "100vw",
          height: "100vh",
          opacity: "var(--SideNavigation-slideIn)",
          backgroundColor: "var(--joy-palette-background-backdrop)",
          transition: "opacity 0.4s",
          transform: {
            xs: "translateX(calc(100% * (var(--SideNavigation-slideIn, 0) - 1) + var(--SideNavigation-slideIn, 0) * var(--Sidebar-width, 0px)))",
            lg: "translateX(-100%)",
          },
        }}
        onClick={() => closeSidebar()}
      />
      <Box sx={{ display: "flex", gap: 1, alignItems: "center", p: 2 }}>
        <IconButton variant="soft" color="primary" size="sm">
          <BrightnessAutoRoundedIcon />
        </IconButton>
        <Typography level="title-lg">Acme Co.</Typography>
        <ColorSchemeToggle sx={{ ml: "auto" }} />
      </Box>
      {/* <Input
        size="sm"
        sx={{ mx: 2 }}
        startDecorator={<SearchRoundedIcon />}
        placeholder="Search"
      /> */}
      <Box
        sx={{
          minHeight: 0,
          overflow: "hidden auto",
          flexGrow: 1,
          display: "flex",
          px: 2,
          flexDirection: "column",
          [`& .${listItemButtonClasses.root}`]: {
            gap: 1.5,
          },
        }}
      >
        <List
          size="sm"
          sx={{
            gap: 1,
            "--List-nestedInsetStart": "30px",
            "--ListItem-radius": (theme) => theme.vars.radius.sm,
          }}
        >
          {items.map((item) => (
            <RenderRootItem key={item.label} {...item} />
          ))}
        </List>

        <List
          size="sm"
          sx={{
            mt: "auto",
            flexGrow: 0,
            "--ListItem-radius": (theme) => theme.vars.radius.sm,
            "--List-gap": "8px",
            mb: 2,
          }}
        >
          <ListItem>
            <ListItemButton>
              <SupportRoundedIcon />
              Support
            </ListItemButton>
          </ListItem>
          <ListItem>
            <ListItemButton>
              <SettingsRoundedIcon />
              Settings
            </ListItemButton>
          </ListItem>
        </List>
        <Card
          invertedColors
          variant="soft"
          color="warning"
          size="sm"
          sx={{ boxShadow: "none" }}
        >
          <Stack direction="row" justifyContent="space-between" alignItems="center">
            <Typography level="title-sm">Used space</Typography>
            <IconButton size="sm">
              <CloseRoundedIcon />
            </IconButton>
          </Stack>
          <Typography level="body-xs">
            Your team has used 80% of your available space. Need more?
          </Typography>
          <LinearProgress variant="outlined" value={80} determinate sx={{ my: 1 }} />
          <Button size="sm" variant="solid">
            Upgrade plan
          </Button>
        </Card>
      </Box>
      <Divider />
      <Box sx={{ display: "flex", gap: 1, alignItems: "center", px: 2, pb: 2 }}>
        <Avatar
          variant="outlined"
          size="sm"
          src="https://images.unsplash.com/photo-1535713875002-d1d0cf377fde?auto=format&fit=crop&w=286"
        />
        <Box sx={{ minWidth: 0, flex: 1 }}>
          <Typography level="title-sm">Siriwat K.</Typography>
          <Typography level="body-xs">siriwatk@test.com</Typography>
        </Box>
        <IconButton size="sm" variant="plain" color="neutral">
          <LogoutRoundedIcon />
        </IconButton>
      </Box>
    </Sheet>
  );
}
