import { ReactNode } from "react";
import { AccessControlContext } from "./AccessControlContext";
import {
  AccessFallbackComponent,
  CanAccessOptions,
  CanAccessResult
} from "./types";
import { useAuthProvider } from "./useAuthProvider";
import { usePermissions } from "./usePermissions";

interface AccessControlProviderProps {
  accessFallback?: AccessFallbackComponent;
  canAccess?: (options: CanAccessOptions) => CanAccessResult;
  children?: ReactNode;
}

export function AccessControlProvider(props: AccessControlProviderProps) {
  const authProvider = useAuthProvider();
  const {
    accessFallback = authProvider?.accessFallback,
    canAccess = authProvider?.canAccess,
    children
  } = props;
  const { permissions } = usePermissions();

  return (
    <AccessControlContext.Provider
      value={{
        accessFallback,
        canAccess,
        permissions
      }}
    >
      {children}
    </AccessControlContext.Provider>
  );
}
