import {
  createElement,
  ReactNode,
  useContext
} from "react";
import { AccessControlContext } from "./AccessControlContext";
import {
  AccessFallbackComponent,
  PermissionIdentifier,
  PermissionTarget
} from "./types";
import { useCan } from "./useCan";

export interface CanAccessProps {
  target: PermissionTarget;
  identifier: PermissionIdentifier;
  fallback?: AccessFallbackComponent;
  children?: ReactNode;
}

export function CanAccess(props: CanAccessProps) {
  const { target, identifier } = props;
  const { can, reason } = useCan(target, identifier);
  const { accessFallback } = useContext(AccessControlContext);
  const { children, fallback = accessFallback } = props;

  if (can) {
    return children;
  }

  if (fallback == null) {
    return null;
  }

  return createElement(fallback, {
    reason,
    target,
    identifier
  })
}
