import { ComponentType } from "react";
import { To } from "react-router-dom";

type QueryFunctionContext = {
  signal?: AbortSignal;
}

export type WithRedirectTo<P = unknown> = P & {
  redirectTo?: To | false;
};

export type AuthRedirectResult = {
  redirectTo?: To | false;
  logoutOnFailure?: boolean;
};

export interface UserIdentity {
  id: string | number;
  fullName?: string;
  email?: string;
  phoneNumber?: string;
  avatarUrl?: string;
  [key: string]: any;
}

export type PermissionIdentifier = string | number;

export type PermissionTarget =
  | 'route'
  | 'menu'
  | 'view'
  | 'element'
  | 'api'
  | string;

export interface Permission {
  identifier: PermissionIdentifier;
  target: PermissionTarget
  [key: string]: any;
}

export interface CanAccessOptions {
  permissions: Permission[];
  target: PermissionTarget;
  identifier: string | number;
}

export interface CanAccessResult {
  can: boolean;
  reason?: string;
}

export type AccessFallbackProps = {
  reason?: string;
  target: PermissionTarget;
  identifier: PermissionIdentifier;
}

export type AccessFallbackComponent = ComponentType<AccessFallbackProps>;

export interface AuthProviderCustoms {}

export interface AuthProvider extends AuthProviderCustoms {
  login: (params: any) => Promise<WithRedirectTo | void | any>;
  logout: (params: any) => Promise<void | false | string>;
  checkAuth: (params: QueryFunctionContext) => Promise<void>;
  checkError: (error: any) => Promise<void>;
  canAccess?: (options: CanAccessOptions) => CanAccessResult;
  accessFallback?: AccessFallbackComponent;
  getIdentity?: (params?: QueryFunctionContext) => Promise<UserIdentity>;
  getPermissions: (params: QueryFunctionContext) => Promise<Permission[]>;
  handleCallback?: (params?: QueryFunctionContext) => Promise<AuthRedirectResult | void | any>;
}
