import {
  useContext,
  useEffect,
  useMemo,
  useState
} from "react";
import {
  CanAccessResult,
  PermissionIdentifier,
  PermissionTarget
} from "./types";
import { AccessControlContext } from "./AccessControlContext";

export function useCan(
  target: PermissionTarget,
  identifier: PermissionIdentifier,
): CanAccessResult {
  const { canAccess, permissions } = useContext(AccessControlContext);
  const [can, setCan] = useState(false);
  const [reason, setReason] = useState<string>();

  useEffect(() => {
    let can = false;
    let reason: string | undefined = "forbidden";
    if (permissions?.length) {
      for (const perm of permissions) {
        if (perm.identifier !== identifier) {
          continue;
        }
        if (perm.target !== target) {
          break;
        }
        if (canAccess) {
          ({ can, reason } = canAccess({
            target,
            permissions,
            identifier,
          }));
        }
      }
    }
    setCan(can)
    setReason(reason)
  }, [
    permissions,
    canAccess,
    target,
    identifier,
  ]);

  return useMemo(() => {
    return {
      can,
      reason,
    }
  }, [can, reason]);
}
