import { ReactNode } from 'react';
import {
  useInRouterContext,
  createHashRouter,
  RouterProvider,
} from 'react-router-dom';
import { BasenameContextProvider } from './BasenameContextProvider';

export interface AppRouterProps {
  basename?: string;
  children: ReactNode;
}

/**
 * Creates a react-router Router unless the app is already inside existing router.
 * Also creates a BasenameContext with the basename prop
 */
export function AdminRouter({ basename = '', children }: AppRouterProps) {
  const isInRouter = useInRouterContext();
  const Router = isInRouter ? DummyRouter : InternalRouter;

  return (
    <BasenameContextProvider basename={isInRouter ? basename : ''}>
      <Router basename={basename}>{children}</Router>
    </BasenameContextProvider>
  );
}

function DummyRouter({ children }: AppRouterProps) {
  return <>{children}</>;
}

function InternalRouter({ basename, children }: AppRouterProps) {
  const router = createHashRouter(
    [{ path: '*', element: <>{children}</> }],
    { basename }
  );
  return <RouterProvider router={router} />;
}
