// tokens are like 'Hello, %{name}'
const defaultTokenRegex = /%\{(.*?)\}/g;

/**
 * Replace tokens by their value in the given string
 *
 * @param {string} template The template with interpolation tokens, e.g. 'Hello, %{name}'
 * @param {object} data The data to interpolate, e.g. { name: 'John' }
 * @returns {string} The interpolated string, e.g. 'Hello, John'
 */
export const substituteTokens = (
  template: string,
  data?: Record<string, string>
): string => {
  if (!template || !data) {
    return template
  }
  return String.prototype.replace.call(
    template,
    defaultTokenRegex,
    function (expression, argument) {
      if (!Object.hasOwn(data, argument)) {
        return expression
      }
      if (data[argument] == null) {
        return expression;
      }
      return data[argument]
    }
  );
};
