import { useCallback, useContext } from "react";
import { useUndoableId } from "./useUndoableId";
import { UndoableContext } from "./UndoableContext";
import { UndoableCallback } from "./types";

export function useUndoableListener() {
  const { listen } = useContext(UndoableContext);
  const id = useUndoableId(true);

  return useCallback((callback: UndoableCallback) => {
    if (id === "") {
       throw new Error('Cannot listen an undoable callback while rendering.');
    }
    listen(id, callback);
  }, [id, listen]);
}
