import { ReactNode, useEffect, useState } from "react";
import { createPortal } from "react-dom";
import { usePortal } from "./usePortal";

export interface PortletProps {
  children?: ReactNode | (() => ReactNode);
  disabled?: boolean;
  localWithPortalNotFound?: boolean;
  to: string;
}

export function Portlet(props: PortletProps) {
  const {
    children,
    disabled,
    to,
    localWithPortalNotFound
  } = props;
  const portalSelector = usePortal(to);

  const [container, setContainer] = useState<Element | null>(() => {
    return typeof portalSelector !== 'string'
      ? (portalSelector || null)
      : typeof document !== 'undefined'
        ? document.querySelector(portalSelector)
        : null;
  });

  // on first mount, we don't have the container yet, so we wait for it
  useEffect(() => {
    setContainer((container) => {
      const isInTheDom = typeof document !== 'undefined' &&
        document.body.contains(container);
      if (container && isInTheDom) {
        return container;
      }
      if (typeof portalSelector !== 'string') {
        return portalSelector || null;
      }
      return typeof document !== 'undefined'
        ? document.querySelector(portalSelector)
        : null;
    });
  }, [portalSelector]);

  if (disabled) {
    return childrenOf(children);
  }

  if (!container) {
    if (localWithPortalNotFound) {
      return childrenOf(children);
    }
    return null;
  }

  return (
    <>
      {
        createPortal(
          childrenOf(children),
          container)
      }
    </>
  );
}

const childrenOf = (v: ReactNode | (() => ReactNode)) => {
  return typeof v === "function" ? v() : v;
};
