import { ReactNode } from "react";
import { RaRecord } from "./types";
import { useRecord } from "./useRecord";

export interface WithRecordProps<TRecord extends RaRecord = RaRecord> {
  render: (record: TRecord) => ReactNode;
  label?: string;
}

/**
 * Render prop version of useRecordContext
 *
 * @example
 * const BookShow = () => (
 *    <Show>
 *       <SimpleShowLayout>
 *          <WithRecord render={record => <span>{record.title}</span>} />
 *      </SimpleShowLayout>
 *   </Show>
 * );
 */
export function WithRecord<TRecord extends RaRecord = RaRecord>(
  { render }: WithRecordProps<TRecord>
) {
  const record = useRecord<TRecord>();
  return record ? <>{render(record)}</> : null;
}
