import { ReactNode } from "react";
import { RaRecord, RecordToStringFunction } from "./types";
import { useRecordContext } from "./useRecordContext";

export interface WithRecordContextProps<TRecord extends RaRecord = RaRecord> {
  render: (
    record: TRecord,
    representation?: string | RecordToStringFunction<TRecord>
  ) => ReactNode;
  label?: string | RecordToStringFunction<TRecord>;
}

/**
 * Render prop version of useRecordContext
 *
 * @example
 * const BookShow = () => (
 *    <Show>
 *       <SimpleShowLayout>
 *          <WithRecord render={record => <span>{record.title}</span>} />
 *      </SimpleShowLayout>
 *   </Show>
 * );
 */
export function WithRecordContext<
  TRecord extends RaRecord = RaRecord
>({ render }: WithRecordContextProps<TRecord>) {
  const context = useRecordContext<TRecord>();

  if (!context) {
    return null;
  }

  return render(
    context.record,
    context.representation,
  );
}
