import { useCallback, useEffect, useRef, useState } from "react";

export function useSafeSetState<T>(
  initialState?: T | (() => T)
): [T | undefined, React.Dispatch<React.SetStateAction<T>>] {
  const [state, setState] = useState(initialState);

  const mountedRef = useRef(false);

  useEffect(() => {
    mountedRef.current = true;
    return () => {
      mountedRef.current = false;
    };
  }, []);

  const safeSetState = useCallback(
    (args: React.SetStateAction<T>) => {
      if (mountedRef.current) {
        return setState(args as React.SetStateAction<T | undefined>);
      }
    },
    [mountedRef, setState]
  );

  return [state, safeSetState];
}
