export type WithSignal<Props> = Props & {
  signal: AbortSignal;
}

export type AsyncExecutor<TData, TVars> = (
  options: WithSignal<TVars>
) => Promise<TData> | PromiseLike<TData> | TData;

export type RetryOptions = {
  count?: number;
  delay?: number;
}

export type RetryResolver<TVars, TError> = (
  failureCount: number,
  error: TError,
  options: WithSignal<TVars>,
) => RetryOptions;

export type ThrowOnError<TVars, TError> = (
  error: TError,
  options: WithSignal<TVars>,
) => boolean;

export type SuccessCallback<TData, TVars> = (
  data: TData,
  options: WithSignal<TVars>,
) => void;

export type ErrorCallback<TVars, TError> = (
  error: TError,
  options: WithSignal<TVars>,
) => void;

export type SettledCallback<TData, TVars, TError> = (
  data: TData | undefined,
  error: TError | undefined,
  options: WithSignal<TVars>,
) => void;

export type ExecuteOptions<TVars> = {
  discard?: () => boolean;
  signal?: AbortSignal;
  variables?: Partial<TVars>;
};

export type ExecuteResult<TData> = {
  abort: () => void;
  signal: AbortSignal;
  promise: Promise<TData | undefined>;
}

export type ExecuteFunction<TData, TVars> = (
  options?: ExecuteOptions<TVars>,
) => ExecuteResult<TData>;
