import { createContext } from "react";
import {
  BodyParams,
  FetchConfig,
  FetchKey,
  HttpMethod,
  SearchParams,
} from "./types";
import { RequestFactory } from "./useCreateRequest";
import { ErrorCorrector } from "./useErrorCorrector";
import { Fetcher } from "./useFetcher";
import { RequestIntercepter } from "./useRequestIntercepter";
import { ResponseTransformer } from "./useResponseTransformer";

export type Combination = 'overrides' | 'chaining';

export type CombinationOptions = {
  searchParams?: Combination;
  headers?: Combination;
  body?: Combination;
};

export interface FormatOptions {
  bool?: (value: boolean) => string; // default to 'true' or 'false'
  null?: string; // default to '' or null
  nan?: string; // default to 'NaN'
  date?: (date: Date) => string; // default to Date.toISOString()
  skipNull?: boolean; // default to true
  skipEmptyString?: boolean; // default to true
  skipNaN?: boolean; // default to true
  arrayFormat?:
  | 'default'       // 'foo=1&foo=2&foo=3'
  | 'bracket'       // 'foo[]=1&foo[]=2&foo[]=3'
  | 'index'         // 'foo[0]=1&foo[1]=2&foo[3]=3'
  | 'dot'           // 'foo.0=1&foo.1=2&foo.2=3'
  | 'comma'         // 'foo=1,2,3'
  | 'line'          // 'foo=1|2|3'
  | 'bracket-comma' // 'foo[]=1,2,3'
  | 'bracket-line'; // 'foo[]=1|2|3'
  objectFormat?:
  | 'bracket' // 'foo[bar]=bazz'
  | 'dot'     // 'foo.bar=bazz'
  | 'colon';  // 'foo:bar=bazz'
  combination?: Combination;
}

export interface BodyMergeConfig extends FormatOptions {
  handle?: (
    bodys: BodyParams[],
    hasFiles: boolean | undefined,
    options: FormatOptions,
  ) => BodyParams;
}

export type BodyBuilder = (
  body: BodyParams,
  hasFiles: boolean,
  options: FormatOptions,
) => BodyInit | null;

export interface SearchParamsMergeConfig extends FormatOptions {
  handleIntoUrl?: (
    url: URL,
    params: SearchParams,
    options: FormatOptions,
  ) => void;
}

export type FetchSubscriber = (
  key: FetchKey,
  handler: (method: HttpMethod, data?: any) => void,
) => VoidFunction;

export type FetchPublisher = (
  key: FetchKey,
  method: HttpMethod,
  data?: any,
) => void;

export interface FetchContextValue {
  baseUrl?: string;
  body?: BodyParams;
  bodyBuilder?: BodyBuilder;
  bodyMergeConfig?: BodyMergeConfig;
  config?: FetchConfig;
  correct?: ErrorCorrector<any, any>;
  createRequest?: RequestFactory;
  fetcher?: Fetcher;
  headers?: HeadersInit;
  headersCombination?: Combination;
  hasFiles?: boolean | undefined;
  intercept?: RequestIntercepter;
  searchParams?: SearchParams;
  searchParamsMergeConfig?: SearchParamsMergeConfig;
  subscribe?: FetchSubscriber;
  publish?: FetchPublisher;
  transform?: ResponseTransformer<any>;
}

export const FetchContext = createContext<FetchContextValue | undefined>(undefined);
