import { useCallback } from "react";
import { Combination } from "./FetchContext";
import { BodyParams } from "./types";
import { useFetchContext } from "./useFetchContext";
import { buildBody, mergeBodys } from "./utils";

export interface UseCreateBodyOptions {
  body?: BodyParams;
  bodyCombination?: Combination;
  hasFiles?: boolean;
  [key: string]: any;
}

export type CreateBodyFunction = (
  body?: BodyParams | undefined,
  hasFiles?: boolean | undefined,
) => BodyInit | null;

export function useCreateBody(options?: UseCreateBodyOptions) {
  const {
    body: bodyFromContext,
    bodyBuilder = buildBody,
    bodyMergeConfig = {},
    hasFiles: hasFilesFromContext,
  } = useFetchContext({});

  return useCallback((
    newBody: BodyParams | undefined,
    newHasFiles: boolean | undefined,
  ): BodyInit | null => {
    const { handle = mergeBodys, ...mergeConfig } = bodyMergeConfig;
    if (options?.bodyCombination) {
      mergeConfig.combination = options.bodyCombination;
    }
    const bodys = toList(bodyFromContext, options?.body, newBody);
    const hasFiles = newHasFiles === true
      || options?.hasFiles === true
      || hasFilesFromContext === true;
    const body = handle(bodys, hasFiles, mergeConfig);
    return bodyBuilder(body, hasFiles, mergeConfig);
  }, [
    bodyBuilder,
    bodyFromContext,
    bodyMergeConfig,
    hasFilesFromContext,
    options?.body,
    options?.bodyCombination,
    options?.hasFiles,
  ]);
}

const toList = <T>(...items: Array<T | undefined>): T[] => {
  return items.filter(i => i !== undefined);
}
