import { useCallback } from "react";
import { Combination } from "./FetchContext";
import { useFetchContext } from "./useFetchContext";

export interface UseCreateHeadersOptions {
  headers?: HeadersInit;
  headersCombination?: Combination;
  [key: string]: any;
}

export type CreateHeadersFunction = (
  headers: HeadersInit | undefined,
) => HeadersInit | undefined;

export function useCreateHeaders(options?: UseCreateHeadersOptions): CreateHeadersFunction {
  const { headers, headersCombination } = useFetchContext({});

  return useCallback((newHeaders: HeadersInit | undefined): HeadersInit | undefined => {
    const combination = options?.headersCombination ?? headersCombination;
    return [
      headers,
      options?.headers,
      newHeaders,
    ].reduce((acc, item) => {
      if (!item) {
        return acc;
      }
      if (combination === 'overrides' && acc) {
        return acc;
      }
      const headers = new Headers(item);
      if (!acc) {
        return headers;
      }
      headers.forEach((value, name) => {
        (acc as Headers).set(name, value);
      });
      return acc;
    });
  }, [
    headers,
    headersCombination,
    options?.headers,
    options?.headersCombination,
  ]);
}
