import { useCallback } from "react";
import { FetchConfig } from "./types";
import { useFetchContext } from "./useFetchContext";

export type RequestFactory = (
  url: URL,
  init: RequestInit,
) => Promise<Request> | Request;

export interface UseCreateRequestOptions {
  createRequest?: RequestFactory;
  [key: string]: any;
}

export type CreateRequestFunction = (
  url: URL,
  config: FetchConfig,
  headers: HeadersInit | undefined,
  body: BodyInit | null | undefined,
  signal?: AbortSignal,
) => Promise<Request> | Request;

const defaultCreateRequest = (url: URL, init: RequestInit) => {
  return new Request(url, init);
};

export function useCreateRequest(options?: UseCreateRequestOptions): CreateRequestFunction {
  const { createRequest } = useFetchContext({});

  return useCallback((
    url: URL,
    config: FetchConfig,
    headers: HeadersInit | undefined,
    body: BodyInit | null | undefined,
    signal: AbortSignal,
  ): Promise<Request> | Request => {
    const handle = options?.createRequest
      ?? createRequest
      ?? defaultCreateRequest;
    return handle(url, {
      ...config,
      headers,
      body,
      signal,
    });
  }, [
    createRequest,
    options?.createRequest,
  ]);
}
