import { useCallback } from "react";
import { Combination } from "./FetchContext";
import { SearchParams } from "./types";
import { useFetchContext } from "./useFetchContext";
import { searchParamsIntoUrl } from "./utils";

export interface UseCreateUrlOptions {
  baseUrl?: string;
  searchParams?: SearchParams;
  searchParamsCombination?: Combination;
  [key: string]: any;
}

export type CreateUrlFunction = (
  url: string,
  searchParams?: SearchParams,
) => URL;

export function useCreateUrl(options?: UseCreateUrlOptions): CreateUrlFunction {
  const {
    baseUrl,
    searchParams,
    searchParamsMergeConfig = {},
  } = useFetchContext({});

  return useCallback((url: string, newSearchParams?: SearchParams): URL => {
    const {
      handleIntoUrl = searchParamsIntoUrl,
      ...mergeConfig
    } = searchParamsMergeConfig;

    const uri = new URL(url, options?.baseUrl || baseUrl);

    if (options?.searchParamsCombination) {
      mergeConfig.combination = options.searchParamsCombination;
    }

    if (searchParams != null) {
      handleIntoUrl(uri, searchParams, mergeConfig);
    }

    if (options?.searchParams != null) {
      handleIntoUrl(uri, options.searchParams, mergeConfig);
    }

    if (newSearchParams != null) {
      handleIntoUrl(uri, newSearchParams, mergeConfig);
    }

    return uri;
  }, [
    baseUrl,
    options?.baseUrl,
    options?.searchParams,
    options?.searchParamsCombination,
    searchParams,
    searchParamsMergeConfig,
  ])
}
