import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";

export type ErrorCorrector<TData, TError> = (
  error: TError,
  request: Request | undefined,
  response: Response | undefined,
) => Promise<TData> | TData;

export interface UseErrorCorrectorOptions<TData, TError> {
  correct?: ErrorCorrector<TData, TError>
  [key: string]: any;
}

function defaultCorrect<TData, TError>(error: TError): Promise<TData> {
  return Promise.reject(error);
}

export function useErrorCorrector<TData, TError>(
  options?: UseErrorCorrectorOptions<TData, TError>,
): ErrorCorrector<TData, TError> {
  const fetchContext = useFetchContext();
  return useCallback((
    error: TError,
    request: Request | undefined,
    response: Response | undefined
  ): Promise<TData> => {
    const handle = options?.correct
      ?? fetchContext?.correct
      ?? defaultCorrect<TData, TError>;
    return handle(error, request, response);
  }, [
    fetchContext?.correct,
    options?.correct,
  ]);
}
