import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";

export type Fetcher = (
  input: RequestInfo | URL,
  init?: RequestInit,
) => Promise<Response>

export interface UseFetcherOptions {
  fetcher?: Fetcher;
    [key: string]: any;
}

export const useFetcher = (options: UseFetcherOptions): Fetcher => {
  const { fetcher } = useFetchContext({});

  return useCallback((
    input: RequestInfo | URL,
    init?: RequestInit,
  ) => {
    const handle = options.fetcher
      ?? fetcher
      ?? fetch;
    return handle(input, init);
  }, [fetcher, options.fetcher]);
}
