import { ExecuteOptions } from "@rakit/use-async";
import { useCallback } from "react";
import {
  FetchVariables,
  useFetch,
  UseFetchOptions,
  UseFetchResult
} from "./useFetch";

export interface UseMutationOptions<TData, TError> extends UseFetchOptions<TData, TError> {}

export type MutationOptions<T extends boolean> = ExecuteOptions<FetchVariables> & {
  returnPromise?: T
}

export type UseMutationResult<TData, TError> = Omit<UseFetchResult<TData, TError>, 'execFetch'> & {
  mutate: <T extends boolean>(
    options?: MutationOptions<T>,
  ) => T extends true ? Promise<TData> : void;
}

export function useMutation<TData = any, TError = unknown>(
  options: UseMutationOptions<TData, TError>,
): UseMutationResult<TData, TError> {
  const {
    ...fetchOptions
  } = options;

  const {
    abort,
    data,
    execFetch,
    error,
    isFetching,
  } = useFetch(fetchOptions);

  const mutate = useCallback(<T extends boolean>({
    returnPromise,
    ...options
  }: MutationOptions<T> = {}): T extends true ? Promise<TData> : void => {
    const res = execFetch(options);
    // @ts-ignore
    return returnPromise === true
      ? res.promise
      : undefined;
  }, [execFetch]);

  return {
    abort,
    data,
    error,
    isFetching,
    mutate,
  };
}
