import { useAutomatic } from "@rakit/use-async";
import { useEffect } from "react";
import { useFetch, UseFetchOptions, UseFetchResult } from "./useFetch";
import { useFetchSubscribe } from "./useFetchSubscribe";

export interface UseQueryOptions<TData = any, TError = unknown> extends UseFetchOptions<TData, TError> {
  disabled?: boolean;
  refetchDelay?: number;
  refetchDelayOnWindowFocused?: number;
  refetchOnWindowFocused?: boolean;
}

export function useQuery<TData = any, TError = unknown>(
  options: UseQueryOptions<TData, TError>,
): UseFetchResult<TData, TError> {
  const {
    disabled = false,
    refetchDelay = 0,
    refetchDelayOnWindowFocused = 0,
    refetchOnWindowFocused,
    ...fetchOptions
  } = options;

  const {
    abort,
    data,
    error,
    execFetch,
    isFetching,
  } = useFetch<TData, TError>(fetchOptions);

  useFetchSubscribe(options.key, (method) => {
    if (method !== 'GET' && !disabled) {
      execFetch();
    }
  }, [disabled, execFetch]);

  useEffect(() => {
    if (disabled) {
      abort();
    }
  }, [disabled, abort]);

  // useEffect(() => {
  //   if (disabled) {
  //     return;
  //   }
  //
  //   let isEffectRunning = true;
  //   let ctrl: ExecuteResult<TData> | undefined;
  //
  //   ;(async () => {
  //     if (refetchDelay > -1) {
  //       await new Promise(resolve => setTimeout(
  //         resolve,
  //         refetchDelay,
  //       ));
  //     }
  //     if (!isEffectRunning) {
  //       return;
  //     }
  //     ctrl = execFetch({
  //       discard: () => !isEffectRunning,
  //     });
  //   })();
  //
  //   return () => {
  //     if (ctrl && !ctrl.signal.aborted) {
  //       ctrl.abort();
  //     }
  //     isEffectRunning = false;
  //   };
  // }, [disabled, execFetch]);
  useAutomatic({
    disabled,
    delay: refetchDelay,
    execute: execFetch,
  });

  // useEffect(() => {
  //   if (disabled ||
  //     !refetchOnWindowFocused ||
  //     typeof window === "undefined") {
  //     return;
  //   }
  //
  //   let isEffectRunning = true;
  //   let ctrl: ExecuteResult<TData> | undefined;
  //
  //   const onFocus = async () => {
  //     if (refetchDelayOnWindowFocused > -1) {
  //       await new Promise(resolve => setTimeout(
  //         resolve,
  //         refetchDelayOnWindowFocused,
  //       ));
  //     }
  //     if (!isEffectRunning) {
  //       return;
  //     }
  //     ctrl = execFetch({
  //       discard: () => !isEffectRunning,
  //     });
  //   };
  //
  //   window.addEventListener("focus", onFocus);
  //
  //   return () => {
  //     if (ctrl && !ctrl.signal.aborted) {
  //       ctrl.abort();
  //     }
  //     isEffectRunning = false;
  //     window.removeEventListener("focus", onFocus);
  //   };
  // }, [
  //   disabled,
  //   refetchDelayOnWindowFocused,
  //   refetchOnWindowFocused,
  //   execFetch,
  // ]);
  useAutomatic({
    disabled: disabled || !refetchOnWindowFocused,
    delay: refetchDelayOnWindowFocused,
    execute: execFetch,
    call: (onFocus: VoidFunction) => {
      if (typeof window !== "undefined") {
        window.addEventListener("focus", onFocus);
        return () => {
          window.removeEventListener("focus", onFocus);
        };
      }
    },
  });

  return {
    abort,
    data,
    error,
    execFetch,
    isFetching,
  };
}
