import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";

export type RequestIntercepter = (request: Request) => Promise<Request> | Request;

export interface UseRequestIntercepterOptions {
  intercept?: RequestIntercepter;
  [key: string]: any;
}

const defaultIntercept = (request: Request) => request;

export function useRequestIntercepter(
  options?: UseRequestIntercepterOptions,
): RequestIntercepter {
  const fetchContext = useFetchContext();
  return useCallback((request: Request) => {
    const handle = options?.intercept
      ?? fetchContext?.intercept
      ?? defaultIntercept
    return handle(request);
  }, [
    options?.intercept,
    fetchContext?.intercept,
  ]);
}
