import { useCallback } from "react";
import { useFetchContext } from "./useFetchContext";

export type ResponseTransformer<TData> = (
  request: Request,
  response: Response,
) => Promise<TData> | TData;

function defaultTransform<TData>(_: Request, response: Response) {
  return response.json() as Promise<TData>;
}

export function useResponseTransformer<TData>(options?: {
  transform?: ResponseTransformer<TData>;
  [key: string]: any;
}): ResponseTransformer<TData> {
  const { transform } = useFetchContext({});
  return useCallback((request: Request, response: Response) => {
    const handle = options?.transform
      ?? transform
      ?? defaultTransform;
    return handle(request, response);
  }, [
    options?.transform,
    transform,
  ]);
}
